/****************************************************************************
**
** Copyright (C) 2010 Laurent Saint-Marcel. All rights reserved.
**
** This file may be used under the terms of the GNU General Public
** License version 3.0 as published by the Free Software Foundation
** and appearing in the file gpl-3.0.txt included in the packaging of
** this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/
// ---------------------------------------------------------------------------
// @file    crc.cpp
// @author  Laurent Saint-Marcel (lstmarcel@yahoo.fr)
// @brief   crc functions
// @web     http://sophiateam.undrgnd.free.fr
// @date    08/08/2010
// ---------------------------------------------------------------------------

// ----------------------------------------------------------------------------
// CRC8
// ----------------------------------------------------------------------------
unsigned char
CRC8(unsigned char data, 
     unsigned char crc)
{
    unsigned char i, feedback;

    for (i=0; i<8; i++)
    {
        feedback = ((crc ^ data) & 0x01);
        if (!feedback) {
            crc >>= 1;
        } else {
            crc ^= 0x18;
            crc >>= 1;
            crc |= 0x80;
        }
        data >>= 1;
    }
    return crc;   
}

// ----------------------------------------------------------------------------
// CRC16
// ----------------------------------------------------------------------------
unsigned char
CRC16(unsigned short data, 
      unsigned char crc)
{
    crc = CRC8(data & 0xFF, crc);
    crc = CRC8((data>>8) & 0xFF, crc);
    return crc;
} 

// ----------------------------------------------------------------------------
// CRC32
// ----------------------------------------------------------------------------
unsigned char
CRC32(unsigned long data, 
      unsigned char crc)
{
    crc = CRC8(data & 0xFF, crc);
    crc = CRC8((data>>8)  & 0xFF, crc);
    crc = CRC8((data>>16) & 0xFF, crc);
    crc = CRC8((data>>24) & 0xFF, crc);
    return crc;
} 
