/****************************************************************************
**
** Copyright (C) 2010 Laurent Saint-Marcel. All rights reserved.
**
** This file may be used under the terms of the GNU General Public
** License version 3.0 as published by the Free Software Foundation
** and appearing in the file gpl-3.0.txt included in the packaging of
** this file. 
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/
// ---------------------------------------------------------------------------
// @file    mainwindow.h
// @author  Laurent Saint-Marcel (lstmarcel@yahoo.fr)
// @brief   Interface between serial port and JTAG/SPI for programming AVR microcontrollers
// @web     http://sophiateam.undrgnd.free.fr
// @date    08/08/2010
// ---------------------------------------------------------------------------

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QtGui/QMainWindow>
#include <QSettings>
#include <QTime>
#include "sirSerial.h"
#include "larvor.h"

class QProgressBar;

namespace Ui
{
    class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();

private:
    void connectSerial();
    void disconnectSerial();
    void updateCOMList();
    void writeCOM(unsigned char data);
    void writeCOM(QList<unsigned char> const& buffer);

private:
    QSettings      settings_;
    QProgressBar   *progress_;
    Ui::MainWindow *ui;
    int            progressMin_;
    int            progressMax_;

    QString        comPort_;
    bool           connected_;
    sirSerial      serialPort_;

    QString        dirname_;
    Larvor         larve_;

    void setProgressLocalRangeMax(int max);
    void setProgressGlobalRange(int max);
    void updateDirName(QString const& filename);
    void loadDirName();
    bool getDeviceInfo(LarvorDevices::DeviceInfo& info, bool dumpInfo=false);

private slots:
    void on_spiRB_clicked();
    void on_jtagRB_clicked();
    void on_actionTestSerialPort_triggered();
    void on_actionCheckEEprom_triggered();
    void on_actionCheckFlash_triggered();
    void on_actionWriteFuse_triggered();
    void on_actionWriteEEprom_triggered();
    void on_actionWriteFlash_triggered();
    void on_actionReadFuse_triggered();
    void on_actionReadEEprom_triggered();
    void on_actionReadFlash_triggered();
    void on_actionGetSignature_triggered();
    void on_actionEraseChip_triggered();

    void progressChanged(double v);
    void on_actionAbout_triggered();
    void on_actionExit_triggered();
    void on_IDCodeBtn_clicked();

private slots:
    // Serial port
    void on_portCB_currentIndexChanged(QString );
    void connected();
    void cannotConnect(QString message);
    void disconnected();
    void dataReceived();

private:
    // Command guard which open the serial port and diplay the command time
    class CommandGuard {
    public:
        CommandGuard(MainWindow* process, 
                     QString const& commandName);
        virtual ~CommandGuard();
        bool testSerialPort();
        void appendInfo(QString info);
        void setStatusSuccessful();

    private:
        MainWindow* process_;
        QTime       chrono_;
        QString     commandName_;
        bool        status_;
    };

};

#endif // MAINWINDOW_H
