/****************************************************************************
**
** Copyright (C) 2007 Laurent Saint-Marcel. All rights reserved.
**
** This file may be used under the terms of the GNU General Public
** License version 3.0 as published by the Free Software Foundation
** and appearing in the file gpl-3.0.txt included in the packaging of
** this file. 
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/
// ---------------------------------------------------------------------------
// @file    sirSerial.cpp
// @author  Laurent Saint-Marcel (lstmarcel@yahoo.fr)
// @web     http://craid.free.fr
// @date    08/05/2001
// ---------------------------------------------------------------------------
// communication on a serial port for windows
// ---------------------------------------------------------------------------


#include <stdio.h>
#include <stdlib.h> 
#include "sirSerial.h"

#ifdef WIN32

// ----------------------------------------------------------------------------
// sirSerial constructor
// ----------------------------------------------------------------------------
sirSerial::sirSerial()
{
    CommPortHandle = 0;
}

// ----------------------------------------------------------------------------
// sirSerial destructor
// ----------------------------------------------------------------------------
sirSerial::~sirSerial()
{
    
}

// ----------------------------------------------------------------------------
// sirSerial::exists
// ----------------------------------------------------------------------------
bool sirSerial::exists(const char * gszPort)
{
    WCHAR gszPor[256];
    HANDLE CommPortHandl;
    for(unsigned int i=0; i <= strlen(gszPort); i++) { // also copy the NULL character at the end
        gszPor[i]= gszPort[i];
    }
    // Code for opening CommPortHandle using CreateFile goes here 
    CommPortHandl = CreateFile( gszPor, 
                                GENERIC_READ | GENERIC_WRITE, 
                                0, 
                                0, 
                                OPEN_EXISTING, 
                                0, 
                                NULL); 
    
    if(CommPortHandl == INVALID_HANDLE_VALUE) {  
        DWORD lastError = GetLastError();
        if (lastError == ERROR_ACCESS_DENIED) {
            return true; // port already used by another application
        }return false;
    }else {
        CloseHandle(CommPortHandl);
        return true;
    }
}

// ----------------------------------------------------------------------------
// sirComThread::open
// ----------------------------------------------------------------------------
bool sirSerial::open(const char * gszPort, 
                     long speed, 
                     int parity, 
                     int bitslength, 
                     int stopbit)
{
    if (strlen(gszPort) <= 0) {
        strcpy(errorMessage_, "Invalid port Name");
        return false;
    }
#if 1
    CommPortTimeOut.ReadIntervalTimeout = MAXDWORD; 
    CommPortTimeOut.ReadTotalTimeoutMultiplier = 0; 
    CommPortTimeOut.ReadTotalTimeoutConstant = 0;
#else
    CommPortTimeOut.ReadIntervalTimeout = 2;
    CommPortTimeOut.ReadTotalTimeoutMultiplier = 1;
    CommPortTimeOut.ReadTotalTimeoutConstant = 1;
#endif
    CommPortTimeOut.WriteTotalTimeoutMultiplier = 0; 
    CommPortTimeOut.WriteTotalTimeoutConstant = 0; 
    
    PortInitSuccessful = 1; 
    // close serial port if already opened
    if(CommPortHandle != 0) close();
    for(unsigned int i=0; i <= strlen(gszPort); i++) { // also copy the NULL character at the end
        gszPort_[i]= gszPort[i];
    }
    
    // Code for opening CommPortHandle using CreateFile goes here 
    CommPortHandle = CreateFile( gszPort_, 
                                 GENERIC_READ | GENERIC_WRITE, 
                                 0, 
                                 0, 
                                 OPEN_EXISTING, 
                                 0, 
                                 NULL); 
    
    if(CommPortHandle == INVALID_HANDLE_VALUE) {  
        DWORD lastError = GetLastError();
        switch(lastError) {
        case ERROR_FILE_NOT_FOUND:
            sprintf(errorMessage_, "Port %s does not exists", gszPort);
            break;
        case ERROR_ACCESS_DENIED:
            sprintf(errorMessage_, "Port %s is used by another application", gszPort);
            break;
        default:
            sprintf(errorMessage_, "Cannot open port %s, errorCode=%d", gszPort, (int)lastError);
            break;
        }
        PortInitSuccessful = 0; 
        return false;
    } else {  //Code implemented only for a valid handle 
        if(!SetCommTimeouts(CommPortHandle, &CommPortTimeOut)) {  
            DWORD lastError = GetLastError();
            switch(lastError) {
            case ERROR_INVALID_FUNCTION:
                sprintf(errorMessage_, "Cannot configure timeout settings of %s ", gszPort);
                break;
            default:
                sprintf(errorMessage_, "Cannot configure timeout settings of %s, errorCode=%d", gszPort, (int)lastError);
                break;
            }
            PortInitSuccessful = 0; 
            return false;
        } 
        
        // clear the memory used by the CommPortDCB object 
        memset(&CommPortDCB,0,sizeof(CommPortDCB)); 
        
        // set DCB 
        CommPortDCB.DCBlength =	sizeof(CommPortDCB); 
        CommPortDCB.BaudRate =	speed; 
        CommPortDCB.Parity =	parity; 
        CommPortDCB.StopBits =	stopbit; 
        CommPortDCB.ByteSize =	bitslength;  
        if(!SetCommState(CommPortHandle, &CommPortDCB)) {  
            DWORD lastError = GetLastError();
            sprintf(errorMessage_, "Cannot configure port %s, errorCode=%d", gszPort, (int)lastError);
				
            PortInitSuccessful = 0;
            return false;
        } 
    } 
    return true;
}

// ----------------------------------------------------------------------------
// sirComThread::close
// ----------------------------------------------------------------------------
bool sirSerial::close()
{
    if(CommPortHandle!=0)
        CloseHandle(CommPortHandle);
    CommPortHandle = 0;
    return true;
}

// ----------------------------------------------------------------------------
// sirComThread::read
// ----------------------------------------------------------------------------
bool sirSerial::read(unsigned char * lpBuf, 
                     unsigned long &length)
{
    if (CommPortHandle == 0 ) 
        return false;
    if( !ReadFile(CommPortHandle, lpBuf, length, &BytesTransferred, NULL) ) {
        return false;
    }
    if(BytesTransferred != length)
        return false;
    return true;
}

// ----------------------------------------------------------------------------
// sirComThread::forceRead
// ----------------------------------------------------------------------------
bool sirSerial::forceRead(unsigned char * lpBuf, 
                          unsigned long &length, 
                          int retry)
{
    if (CommPortHandle == 0 ) 
        return false;
    unsigned long l2 = 0;
    while(l2 != length && retry != -1) {
        if( !ReadFile(CommPortHandle, lpBuf+l2, length-l2, &BytesTransferred, NULL) ) {
            return false;
        }
        l2 += BytesTransferred;
        --retry;
        if (retry<-1) retry=-2;
        if (l2 != length && retry != -1)
            usleep(10);
    }
    //length = BytesTransferred;
#ifdef SIR_SERIAL_DEBUG
    FILE* fptr=fopen("com.txt", "a");
    for(unsigned int i=0; i < l2; ++i) {
        fprintf(fptr, "0x%2.2x ", lpBuf[i]);
    }
    fclose(fptr);
#endif
    if (l2 == length)
        return true;
    else
        return false;
}

// ----------------------------------------------------------------------------
// sirComThread::write
// ----------------------------------------------------------------------------
bool sirSerial::write(unsigned char * lpBuf, 
                      unsigned long length)
{
    if (CommPortHandle == 0 ) 
        return false;
    if( !WriteFile(CommPortHandle, lpBuf, length, &BytesTransferred, NULL) ) {
        return false;
    }
    if(BytesTransferred != length)
        return false;
    return true;
}
// ----------------------------------------------------------------------------
// sirComThread::write
// ----------------------------------------------------------------------------
bool sirSerial::write(unsigned char buf)
{
    unsigned long length=1;
    unsigned char lbuf[1];
    lbuf[0] = buf;
    return write(lbuf, length);
}

#endif  //WIN32

SerialPort::SerialPort(QObject *parent)
    : QThread(parent),
      serial_(NULL),
      request_(sircNotConnected),
      portName_("COM1"),
      baudRate_(4800),
      needHasDataSignal_(true),
      isConnected_(false)
{
}

SerialPort::~SerialPort()
{
    delete serial_;
}

bool SerialPort::read(QList<unsigned char> & buffer)
{
    QMutexLocker locker(&mutex_);
    if (!readBuffer_.empty()) {
        buffer += readBuffer_;
        readBuffer_.clear();
        needHasDataSignal_=true; // while emit signal when new data arrive
        return true;
    }
    return false;
}

bool SerialPort::write(QList<unsigned char> const& buffer)
{
    QMutexLocker locker(&mutex_);
    if (request_ == sircConnected) {
        QList<unsigned char>::const_iterator it;
        for(it = buffer.begin(); it != buffer.end(); ++it) {
            writeBuffer_.push_back(*it);
        }
        return true;
    } else {
        return false;
    }
}

bool SerialPort::write(unsigned char data)
{

#ifdef SERIAL_PORT_DEBUG
    FILE*fptr=fopen("debug.log", "a");
    fprintf(fptr, "\nSerialPort::write:%x before lock", data);
    fclose(fptr);
#endif
    QMutexLocker locker(&mutex_);
    if (request_ == sircConnected) {
        writeBuffer_.push_back(data);
#ifdef SERIAL_PORT_DEBUG
        FILE*fptr=fopen("debug.log", "a");
        fprintf(fptr, "\nwrite:%x", data);
        fclose(fptr);
#endif
        return true;
    } else {
        return false;
    }
}

// ----------------------------------------------------------------------------
// sirComThread::connect
// ----------------------------------------------------------------------------
// order the opening of the serial port connection. When the connection is 
// opened, the signal connected() is emitted
// ----------------------------------------------------------------------------
void SerialPort::open(QString const& port,
                      int baudRate)
{
    portName_ = port;
    baudRate_ = baudRate;
    readBuffer_.clear();
    writeBuffer_.clear();
    needHasDataSignal_=true;
    setRequest(sircToConnect);
}

// ----------------------------------------------------------------------------
// sirComThread::disconnect
// ----------------------------------------------------------------------------
// order the closing of the serial port connection. When the connection is 
// closed, the signal disconnected() is emitted
// ----------------------------------------------------------------------------
void SerialPort::close()
{
    setRequest(sircToDisconnect);
}

// ----------------------------------------------------------------------------
// sirComThread::killThread
// ----------------------------------------------------------------------------
// order the destruction of the thread. Use sirComThread::wait() to wait for the 
// thread to terminate
// ----------------------------------------------------------------------------
void SerialPort::killThread()
{
    setRequest(sircToKill);
}

// ----------------------------------------------------------------------------
// sirComThread::getMode
// ----------------------------------------------------------------------------
// return the latest requested Mode for the thread 
// ----------------------------------------------------------------------------
SerialPort::ThreadRequest SerialPort::getMode()
{
    QMutexLocker locker(&mutex_);
    return request_;
}

// ----------------------------------------------------------------------------
// sirComThread::setRequest
// ----------------------------------------------------------------------------
// set requested Mode for the thread 
// ----------------------------------------------------------------------------
bool SerialPort::setRequest(SerialPort::ThreadRequest r)
{
    QMutexLocker locker(&mutex_);
    request_ = r;
    return true;
}

void SerialPort::run() 
{
    while(true) {
        switch(getMode()) {
        case sircNotConnected:
            msleep(10);
            continue;
        case sircToConnect:
            {
                char txt[50];
                strcpy(txt, portName_.toAscii());
                QMutexLocker locker(&mutex_);
                if (serial_) {
                    serial_->close();
                } else {
                    serial_ = new sirSerial();
                }
                if (!serial_ || !serial_->open(portName_.toAscii(), baudRate_)) {
                    request_ = sircNotConnected;
		    locker.unlock();
		    isConnected_ = false;	
                    emit cannotConnect(serial_?serial_->getErrorMessage():"");
                } else {
                    request_ = sircConnected;
                    locker.unlock();
		    isConnected_ = true;
                    emit connected();
                }
            }
            break;
        case sircConnected:
            msleep(10);
            break;
        case sircToDisconnect:
            {
                QMutexLocker locker(&mutex_);
                if (serial_) {
                    serial_->close();
                }
                request_ = sircNotConnected;
                locker.unlock();
                isConnected_ = false;	
                emit disconnected();
            }
            continue;
        case sircToKill:
            return; 
        case sircWrite:
            break;
        default:
            break;
        }
        serialWrite();
        serialRead();
    }
}

void SerialPort::serialWrite()
{
    if (serial_ && !writeBuffer_.empty()) {
        QMutexLocker locker(&mutex_);
#ifdef SERIAL_PORT_DEBUG
        FILE*fptr=fopen("debug.log", "a");
        fprintf(fptr, "\n>");
#endif
        unsigned char msg[2];
        while(!writeBuffer_.empty()) {
            msg[0] = writeBuffer_.front();
#ifdef SERIAL_PORT_DEBUG
            fprintf(fptr, "0x%x ", msg[0]);
#endif
            if(serial_->write(msg, 1)) {
                writeBuffer_.pop_front();
            } else {
                break;
            }
        }
#ifdef SERIAL_PORT_DEBUG
        fclose(fptr);
#endif
    }
}

void SerialPort::serialRead()
{
    if (serial_) {
        QMutexLocker locker(&mutex_);
        bool hasData=false;
        unsigned long l=1;
        unsigned char msg[2];
        msg[0]=0x00;
#ifdef SERIAL_PORT_DEBUG
        FILE*fptr=NULL;
#endif
        while(serial_->read(msg, l)) {
#ifdef SERIAL_PORT_DEBUG

            if (!fptr) {
                fptr=fopen("debug.log", "a");
                fprintf(fptr, "\n<");
            }
            fprintf(fptr, "0x%x ", msg[0]);
#endif
            readBuffer_.push_back(msg[0]);
            l=1;
            hasData = true;
        }
#ifdef SERIAL_PORT_DEBUG
        if(fptr) {
            fclose(fptr);
        }
#endif
        locker.unlock();
        if (hasData && needHasDataSignal_) {
            needHasDataSignal_ = false;
            emit dataReceived();
        }
    }
}
