/****************************************************************************
**
** Copyright (C) 2010 Laurent Saint-Marcel. All rights reserved.
**
** This file may be used under the terms of the GNU General Public
** License version 3.0 as published by the Free Software Foundation
** and appearing in the file gpl-3.0.txt included in the packaging of
** this file. 
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/
// ---------------------------------------------------------------------------
// @file    writeFuse.cpp
// @author  Laurent Saint-Marcel (lstmarcel@yahoo.fr)
// @brief   Interface between serial port and JTAG/SPI for programming AVR microcontrollers
// @web     http://sophiateam.undrgnd.free.fr
// @date    08/08/2010
// ---------------------------------------------------------------------------

#include "writeFuse.h"

WriteFuseDialog::WriteFuseDialog(QWidget   * parent,
                                 LarvorDevices::DeviceInfo const& device,
								 unsigned char fuseHigh,
                                 unsigned char fuseLow,
                                 unsigned char fuseExtended):
    QDialog(parent),
    fuseHigh_(fuseHigh),
    fuseLow_(fuseLow),
    fuseExtended_(fuseExtended),
    fuseMode_(highFuseMode),
	device_(device)
{
     ui_.setupUi(this);

     connect(ui_.highFuseRB, SIGNAL(toggled(bool)),
             this, SLOT(fuseModeChanged()));
     connect(ui_.lowFuseRB, SIGNAL(toggled(bool)),
             this, SLOT(fuseModeChanged()));
     connect(ui_.extendedFuseRB, SIGNAL(toggled(bool)),
             this, SLOT(fuseModeChanged()));

     connect(ui_.valueLE, SIGNAL(textEdited ( const QString &)),
             this, SLOT(fuseValueChanged()));

     ui_.lowFuseRB->setChecked(true);
     fuseModeChanged();
}

WriteFuseDialog::~WriteFuseDialog()
{
}

void WriteFuseDialog::fuseModeChanged()
{
    QString txt;
    if (ui_.highFuseRB->isChecked()) {
        fuseMode_ = highFuseMode;
    } else if (ui_.lowFuseRB->isChecked()) {
        fuseMode_ = lowFuseMode;
    } else {
        fuseMode_ = extendedFuseMode;
    }
    ui_.textEdit->setText(txt);
    updateFuseValue();
    updateFuseDescription();
}


void WriteFuseDialog::fuseValueChanged()
{
    bool ok=false;
    unsigned long long v=0;
    QString txt = ui_.valueLE->text();
    if (txt.startsWith("0x")) {
        txt.replace("0x", "");
        v = txt.toInt(&ok, 16);
        ok &=  (v >= 0 && v <= 0xFF);
    }
    if (!ok) {
        updateFuseValue();
    } else {
        unsigned char fuse = v;
        switch(fuseMode_) {
        case highFuseMode:
            fuseHigh_ = fuse;
            break;
        case lowFuseMode:
            fuseLow_ = fuse;
            break;
        case extendedFuseMode:
            fuseExtended_ = fuse;
            break;
        }
        updateFuseDescription();
    }
}

void WriteFuseDialog::updateFuseValue()
{
    int fuse=0;
    switch(fuseMode_) {
        case highFuseMode:
            fuse = fuseHigh_;
            break;
        case lowFuseMode:
            fuse = fuseLow_;
            break;
        case extendedFuseMode:
            fuse = fuseExtended_;
            break;
    }
    ui_.valueLE->setText("0x" + QString::number(fuse, 16));
}

void WriteFuseDialog::updateFuseDescription()
{
    QString txt;
    switch(fuseMode_) {
        case highFuseMode:
            txt = LarvorDevices::fuseHighToString(device_, fuseHigh_);
            break;
        case lowFuseMode:
            txt = LarvorDevices::fuseLowToString(device_, fuseLow_);
            break;
        case extendedFuseMode:
            txt = LarvorDevices::fuseExtendedToString(device_, fuseExtended_);
            break;
    }
    ui_.textEdit->setText(txt);
}
