/****************************************************************************
**
** Copyright (C) 2010 Laurent Saint-Marcel. All rights reserved.
**
** This file may be used under the terms of the GNU General Public
** License version 3.0 as published by the Free Software Foundation
** and appearing in the file gpl-3.0.txt included in the packaging of
** this file. 
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/
// ---------------------------------------------------------------------------
// @file    writeFuse.h
// @author  Laurent Saint-Marcel (lstmarcel@yahoo.fr)
// @brief   Interface between serial port and JTAG/SPI for programming AVR microcontrollers
// @web     http://sophiateam.undrgnd.free.fr
// @date    08/08/2010
// ---------------------------------------------------------------------------

#ifndef WRITEFUSE_H
#define WRITEFUSE_H

#include <QDialog>
#include "ui_fuse.h"
#include "Larvor.h"

class WriteFuseDialog : public QDialog
{
   Q_OBJECT

  public:
    enum FuseMode {
          highFuseMode,
          lowFuseMode,
          extendedFuseMode
    };

    WriteFuseDialog(QWidget   * parent,
					LarvorDevices::DeviceInfo const& device_,
                    unsigned char fuseHigh,
                    unsigned char fuseLow,
                    unsigned char fuseExtended);

    virtual ~WriteFuseDialog();

    FuseMode      getFuseMode() const     { return fuseMode_; }
    unsigned char getFuseHigh() const     { return fuseHigh_; }
    unsigned char getFuseLow() const      { return fuseLow_; }
    unsigned char getFuseExtended() const { return fuseExtended_; }


 public slots:
    void fuseModeChanged();
    void fuseValueChanged();

 private:
    void updateFuseValue();
    void updateFuseDescription();

 private:
    Ui::FuseDialog  ui_;
    unsigned char   fuseHigh_;
    unsigned char   fuseLow_;
    unsigned char   fuseExtended_;
    FuseMode        fuseMode_;
	LarvorDevices::DeviceInfo const& device_;
};

#endif // WRITEFUSE_H
